package WebService::Cmis::Policy;

=head1 NAME

WebService::Cmis::Policy - Representation of a cmis policy

=head1 DESCRIPTION

TODO: this isn't implemented yet

=cut

use strict;
use warnings;
use WebService::Cmis::Object ();
our @ISA = ('WebService::Cmis::Object');

=head1 METHODS

=over 4

=back

=head1 COPYRIGHT AND LICENSE

Copyright 2012 Michael Daum

This module is free software; you can redistribute it and/or modify it under
the same terms as Perl itself.  See F<http://dev.perl.org/licenses/artistic.html>.

=cut

1;
